@@ -2,7 +2,7 @@
 
 
 ###############################################################################
-# Copyright (c) 2020 Riverbank Computing Limited.
+# Copyright (c) 2022 Riverbank Computing Limited.
 ###############################################################################
 
 
@@ -69,6 +69,12 @@ try:
 except ImportError:
     pass
 
+try:
+    import PyQt5.QtNetworkAuth
+    optional_products.append("PyQtNetworkAuth")
+except ImportError:
+    pass
+
 try:
     import PyQt5.QtPurchasing
     optional_products.append("PyQtPurchasing")
