@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,6 +26,7 @@
 
 import csv
 import glob
+from importlib import resources
 import os
 import shlex
 import shutil
@@ -41,6 +42,10 @@ from ..user_exception import UserException
 from ..version import PYQTDEPLOY_HEXVERSION
 from ..version_number import VersionNumber
 
+from . import lib as lib_package
+from .lib import bootstrap as bootstrap_package
+from .lib import bootstrap_external as bootstrap_external_package
+
 
 class Builder:
     """ The builder for a project. """
@@ -141,9 +146,9 @@ class Builder:
         # original source.  We continue to use a local copy of _bootstrap.py
         # as it still needs to be frozen and we don't want to depend on an
         # external source.
-        self._freeze_bootstrap('bootstrap', self._build_dir, job_writer,
+        self._freeze_bootstrap(bootstrap_package, self._build_dir, job_writer,
                 python)
-        self._freeze_bootstrap('bootstrap_external', self._build_dir,
+        self._freeze_bootstrap(bootstrap_external_package, self._build_dir,
                 job_writer, python)
 
         # Freeze any main application script.
@@ -177,7 +182,7 @@ class Builder:
         """
 
         lib_dir = self._sysroot.target_lib_dir
-        lib_so = []
+        shlibs = []
 
         for value in libs:
             if value.startswith('-L'):
@@ -186,10 +191,13 @@ class Builder:
                 if not os.path.isabs(lib_dir):
                     lib_dir = os.path.join(self._sysroot.sysroot_dir, lib_dir)
             elif value.startswith('-l'):
-                lib_so.append('lib' + value[2:] + '.so')
+                if self._target.platform.name == 'win':
+                    shlibs.append(value[2:] + '.dll')
+                else:
+                    shlibs.append('lib' + value[2:] + '.so')
 
         if lib_dir != '':
-            for lib in lib_so:
+            for lib in shlibs:
                 lib_path = os.path.realpath(os.path.join(lib_dir, lib))
 
                 if not os.path.isfile(lib_path):
@@ -263,6 +271,12 @@ class Builder:
 
             used_values.add(value)
 
+    def _copy_to_build_dir(self, name):
+        """ Copy a file resource to the build directory. """
+
+        with resources.path(lib_package, name) as path:
+            shutil.copy2(path, self._build_dir)
+
     @staticmethod
     def _freeze(job_writer, label, out_file, in_file, name, as_c=False):
         """ Freeze a Python source file to a C header file or a data file. """
@@ -275,16 +289,18 @@ class Builder:
 
         job_writer.writerow([label, out_file, in_file, name, conversion])
 
-    def _freeze_bootstrap(self, name, build_dir, job_writer, python):
+    def _freeze_bootstrap(self, package, build_dir, job_writer, python):
         """ Freeze a version dependent bootstrap script. """
 
         # Find the bootstrap script appropriate for this version of Python.
-        bootstrap_dir = self._get_lib_path(name)
         bootstrap = None
         bootstrap_version = None
 
-        for fn in os.listdir(bootstrap_dir):
-            version = fn.split('-')[-1]
+        for fn in resources.contents(package):
+            if fn.startswith('__'):
+                continue
+
+            name, version = fn.split('-')
             if version.endswith('.py'):
                 version = version[:-3]
 
@@ -304,10 +320,10 @@ class Builder:
 
         assert bootstrap is not None
 
-        bootstrap_path = os.path.join(bootstrap_dir, bootstrap)
-        self._freeze(job_writer, bootstrap,
-                os.path.join(build_dir, 'frozen_' + name + '.h'),
-                bootstrap_path, 'pyqtdeploy_' + name, as_c=True)
+        with resources.path(package, bootstrap) as path:
+            self._freeze(job_writer, bootstrap,
+                    os.path.join(build_dir, 'frozen_' + name + '.h'), path,
+                    'pyqtdeploy_' + name, as_c=True)
 
     def _generate_resources(self, parts, job_writer, nr_resources):
         """ Generate the application resource files and return the names of
@@ -366,13 +382,6 @@ class Builder:
 
         return abs_resource_path
 
-    @staticmethod
-    def _get_lib_path(name):
-        """ Get the pathname of a file or directory in the 'lib' sub-directory.
-        """
-
-        return os.path.join(os.path.dirname(__file__), 'lib', name)
-
     def _run_freeze(self, python, job_filename, opt):
         """ Run the accumlated freeze jobs. """
 
@@ -383,10 +392,12 @@ class Builder:
         elif opt == 1:
             args.append('-O')
 
-        args.append(self._get_lib_path('freeze.py'))
-        args.append(job_filename)
+        with resources.path(lib_package, 'freeze.py') as path:
+            args.append(str(path))
+            args.append(job_filename)
 
-        self._host.platform.run(*args, message_handler=self._message_handler)
+            self._host.platform.run(*args,
+                    message_handler=self._message_handler)
 
     def _write_inittab(self, f, inittab, c_inittab):
         """ Write the Python version specific extension module inittab. """
@@ -637,6 +648,9 @@ int main(int argc, char **argv)
         used_includepath.add(self._sysroot.target_include_dir)
         used_includepath.add(python.target_py_include_dir)
 
+        if python.version >= (3, 9):
+            used_includepath.add(python.target_py_include_dir + '/internal')
+
         used_libs.add('-L' + self._sysroot.target_lib_dir)
         used_libs.add('-l' + python.target_py_lib)
 
@@ -696,9 +710,9 @@ int main(int argc, char **argv)
         if qmake_config:
             f.write('CONFIG += %s\n' % ' '.join(qmake_config))
 
-        # Python v3.6.0 requires C99 at least.  Note that specifying 'c++11' in
+        # Python requires C99 at least.  Note that specifying 'c++11' in
         # 'CONFIG' doesn't affect 'CFLAGS'.
-        if python.version >= (3, 6) and target_platform != 'win':
+        if target_platform != 'win':
             f.write('\n')
             f.write('QMAKE_CFLAGS += -std=c99\n')
 
@@ -738,10 +752,8 @@ int main(int argc, char **argv)
         f.write('SOURCES = pyqtdeploy_main.cpp pyqtdeploy_start.cpp pdytools_module.cpp\n')
         self._write_used_values(f, used_sources, 'SOURCES')
         self._write_main(used_inittab, used_defines)
-        shutil.copy2(self._get_lib_path('pyqtdeploy_start.cpp'),
-                self._build_dir)
-        shutil.copy2(self._get_lib_path('pdytools_module.cpp'),
-                self._build_dir)
+        self._copy_to_build_dir('pyqtdeploy_start.cpp')
+        self._copy_to_build_dir('pdytools_module.cpp')
 
         f.write('\n')
         f.write('HEADERS = {0}\n'.format(' '.join(headers)))
@@ -751,25 +763,28 @@ int main(int argc, char **argv)
             f.write('\n')
             self._write_used_values(f, used_libs, 'LIBS')
 
-        # Additional configuration for Android.
+        # Additional platform=specific configuration.
         if target_platform == 'android':
             f.write('\n')
             f.write('ANDROID_ABIS = {}\n'.format(self._target.android_abi))
 
-            if bundled_shared_libs and target_platform == 'android':
+            if bundled_shared_libs:
                 f.write(
                         'ANDROID_EXTRA_LIBS += %s\n' % ' '.join(
                                 bundled_shared_libs))
+        elif target_platform == 'win':
+            # If we are using the installed Python on Windows then copy in the
+            # required DLLs.
+            if used_dlls:
+                self._copy_python_dlls(python, used_dlls, f)
 
-        # If we are using the installed Python on Windows then copy in the
-        # required DLLs.
-        if used_dlls:
-            self._copy_windows_dlls(python, used_dlls, f)
+            # Copy in any component DLLs.
+            if bundled_shared_libs:
+                self._copy_dlls(bundled_shared_libs, f)
 
         # Add the project independent post-configuration stuff.
-        with open_file(self._get_lib_path('post_configuration.pro')) as pro_f:
-            f.write('\n')
-            f.write(pro_f.read())
+        f.write('\n')
+        f.write(resources.read_text(lib_package, 'post_configuration.pro'))
 
         # Add any application specific stuff.
         qmake_configuration = project.qmake_configuration.strip()
@@ -832,9 +847,26 @@ int main(int argc, char **argv)
 
             f.write('{0} += {1}\n'.format(qmake_var, value))
 
-    def _copy_windows_dlls(self, python, parts, f):
-        """ Generate additional qmake commands to install additional Windows
-        DLLs so that the application will be able to run.
+    def _copy_dlls(self, dlls, f):
+        """ Generate additional qmake commands to install additional DLLs so
+        that the application will be able to run.
+        """
+
+        for dll in dlls:
+            f.write('''
+PDY_DLL = %s
+exists($$PDY_DLL) {
+    CONFIG(debug, debug|release) {
+        QMAKE_POST_LINK += $(COPY_FILE) $$shell_path($$PDY_DLL) $$shell_path($$OUT_PWD/debug) &
+    } else {
+        QMAKE_POST_LINK += $(COPY_FILE) $$shell_path($$PDY_DLL) $$shell_path($$OUT_PWD/release) &
+    }
+}
+''' % dll)
+
+    def _copy_python_dlls(self, python, parts, f):
+        """ Generate additional qmake commands to install additional DLLs from
+        the Python installation so that the application will be able to run.
         """
 
         dlls = []
@@ -857,14 +889,6 @@ int main(int argc, char **argv)
         if python.version >= (3, 8):
             dlls.append('vcruntime140_1.dll')
 
-        for name in dlls:
-            f.write('''
-PDY_DLL = %s\\%s
-exists($$PDY_DLL) {
-    CONFIG(debug, debug|release) {
-        QMAKE_POST_LINK += $(COPY_FILE) $$shell_path($$PDY_DLL) $$shell_path($$OUT_PWD/debug) &
-    } else {
-        QMAKE_POST_LINK += $(COPY_FILE) $$shell_path($$PDY_DLL) $$shell_path($$OUT_PWD/release) &
-    }
-}
-''' % (python.target_lib_dir, name))
+        dlls = [os.path.join(python.target_lib_dir, dll) for dll in dlls]
+
+        self._copy_dlls(dlls, f)
