@@ -93,23 +93,25 @@ class Platform:
 
         try:
             with subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True) as process:
-                try:
-                    while process.poll() is None:
-                        line = process.stdout.readline()
-                        if not line:
-                            continue
+                if capture:
+                    stdout, _ = process.communicate()
+                    stdout = stdout.strip()
+                else:
+                    stdout = None
+
+                    try:
+                        while process.poll() is None:
+                            line = process.stdout.readline()
+                            if not line:
+                                continue
 
-                        if capture:
-                            stdout.append(line)
-                        else:
                             message_handler.verbose_message(line.rstrip())
 
-                    if process.returncode != 0:
-                        detail = "returned exit code {}".format(
-                                process.returncode)
+                    except Exception as e:
+                        process.kill()
 
-                except Exception as e:
-                    process.kill()
+                if process.returncode != 0:
+                    detail = "returned exit code {}".format(process.returncode)
         except Exception as e:
             detail = str(e)
 
@@ -117,7 +119,7 @@ class Platform:
             raise UserException(
                     "execution of '{0}' failed: {1}".format(args[0], detail))
 
-        return ''.join(stdout).strip() if capture else None
+        return stdout
 
     def verify_as_target(self, message_handler):
         """ Verify the platform as a target. """
@@ -145,7 +147,7 @@ class Architecture:
         """
 
         if name is None:
-            from distutils.util import get_platform
+            from sysconfig import get_platform
 
             parts = get_platform().split('-')
             base_platform = parts[0]
@@ -155,7 +157,7 @@ class Architecture:
                 size = '64' if parts[1] == 'x86_64' else '32'
             elif base_platform == 'macosx':
                 name = 'macos'
-                size = '64' if parts[2] == 'x86_64' else '32'
+                size = '64' if parts[2] in ('x86_64', 'universal2') else '32'
             elif base_platform in ('win32', 'win'):
                 name = 'win'
 
@@ -269,26 +271,22 @@ class ApplePlatform(Platform):
 
         super().verify_as_target(message_handler)
 
+        # Get the SDK path and version number.
         self.apple_sdk = self.run('xcrun', '--sdk', self.sdk_name,
                 '--show-sdk-path', message_handler=message_handler,
                 capture=True)
 
-        if not self.apple_sdk:
+        self.apple_sdk_version = self.run('xcrun', '--sdk', self.sdk_name,
+                '--show-sdk-version', message_handler=message_handler,
+                capture=True)
+
+        if not self.apple_sdk or not self.apple_sdk_version:
             raise UserException(
                     "a valid '{0}' SDK could not be found".format(
                             self.sdk_name))
 
-        # Parse the version number.
-        version_str = os.path.basename(self.apple_sdk)
-
-        if version_str.startswith(self.sdk_prefix):
-            version_str = version_str[len(self.sdk_prefix):]
-
-        if version_str.endswith('.sdk'):
-            version_str = version_str[:-len('.sdk')]
-
         self.apple_sdk_version = VersionNumber.parse_version_number(
-                version_str)
+                self.apple_sdk_version)
 
 
 # Define and implement the different platforms and architectures.  These should
