@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,10 @@ class AbstractComponent(ABC):
     # The dict of parts provided by the component.
     provides = {}
 
+    # This is set if the version number is optional.  If it is then it must be
+    # set by the component's verify() method.
+    version_is_optional = False
+
     @property
     def android_abi(self):
         """ The Android ABI being used. """
@@ -240,7 +244,8 @@ class AbstractComponent(ABC):
         configurable options.
         """
 
-        return [ComponentOption('version', required=True,
+        return [ComponentOption('version',
+                required=not self.version_is_optional,
                 help="The version number of the component.")]
 
     def get_python_install_path(self, major, minor):
@@ -437,7 +442,7 @@ class AbstractComponent(ABC):
         message = "v{0} is unsupported".format(self.version)
 
         if detail is not None:
-            message += ' ' + detail
+            message += ', ' + detail
 
         self.error(message)
 
@@ -524,9 +529,14 @@ class AbstractComponent(ABC):
         if unused:
             self.error("unknown option(s): {0}".format(', '.join(unused)))
 
-        # Allow the version number to be defined by an environment variable.
-        self.version = self.parse_version_number(
-                os.path.expandvars(self.version))
+        # Parse any version number.
+        if self.version == '':
+            self.version = None
+        else:
+            # Allow the version number to be defined by an environment
+            # variable.
+            self.version = self.parse_version_number(
+                    os.path.expandvars(self.version))
 
     @staticmethod
     def add_to_path(name):
@@ -571,6 +581,9 @@ class AbstractComponent(ABC):
             self._install_status = self._IS_INSTALLED
             manifest[self.name] = self.version
 
+            # Checkpoint the manifest.
+            self._sysroot.write_manifest(manifest)
+
         elif self._install_status == self._IS_IN_PROGRESS:
             self.error("the component is part of a circular dependency")
 
@@ -592,8 +605,6 @@ class AbstractComponent(ABC):
         if self._parts is None:
             self._parts = {}
 
-            openssl = self.get_component('OpenSSL', required=False)
-
             # Get the provided version and target-specific parts.
             provides = {}
             for unscoped_name, versions in self.provides.items():
@@ -604,7 +615,7 @@ class AbstractComponent(ABC):
             # For each provided part remember the part or None if any of its
             # dependencies are unavailable.
             for name, part in provides.items():
-                self._add_part(name, part, openssl, provides)
+                self._add_part(name, part, provides)
 
             # Remove all unavailable parts.
             self._parts = {n: p for n, p in self._parts.items()
@@ -622,7 +633,7 @@ class AbstractComponent(ABC):
         # site-packages directory.
         return self.get_component('Python').target_sitepackages_dir
 
-    def _add_part(self, name, part, openssl, provides):
+    def _add_part(self, name, part, provides):
         """ Add a part if all its dependencies are available. """
 
         # Handle the trivial case.
@@ -653,7 +664,7 @@ class AbstractComponent(ABC):
                 dep_name = Part.get_name(component_name, part_name[1:])
             elif part_name.startswith('!'):
                 # This is only provided if OpenSSL is not available.
-                if openssl is not None:
+                if self.get_component('OpenSSL', required=False) is not None:
                     continue
 
                 dep_name = Part.get_name(component_name, part_name[1:])
@@ -668,7 +679,7 @@ class AbstractComponent(ABC):
                         part = None
                         break
                 else:
-                    self._add_part(dep_name, dep_part, openssl, provides)
+                    self._add_part(dep_name, dep_part, provides)
 
                     # See if the dependency was actually available.
                     if self._parts[dep_name] is None:
@@ -718,42 +729,32 @@ class AbstractComponent(ABC):
         All target-specific values are resolved.
         """
 
-        if not isinstance(versions, tuple):
-            versions = (versions, )
+        # Note that we can't check the Android API level because we don't yet
+        # know which level is being targeted.
+        part = Part.get_applicable_part(self.name, unscoped_name, versions,
+                self._sysroot.target, self.version)
 
-        # Go through each version of the part.
-        for part in versions:
-            if part.applies_to(self.version):
-                # Discard parts for other targets.  Note that we can't check
-                # the Android API level because we don't yet know which level
-                # is being targeted.
-                if part.target != '' and not self._sysroot.target.is_targeted(part.target):
-                    part = None
-                    break
+        if part is None:
+            return None
 
-                # Don't modify the original part.
-                part = copy.deepcopy(part)
+        # Don't modify the original part.
+        part = copy.deepcopy(part)
 
-                # Save the scoped name of the part.
-                part.name = Part.get_name(self.name, unscoped_name)
+        # Save the scoped name of the part.
+        part.name = Part.get_name(self.name, unscoped_name)
 
-                # Normalise the dependencies.
-                part.deps = self._normalised_deps(part.deps)
-                part.hidden_deps = self._normalised_deps(part.hidden_deps)
+        # Normalise the dependencies.
+        part.deps = self._normalised_deps(part.deps)
+        part.hidden_deps = self._normalised_deps(part.hidden_deps)
 
-                # Resolve all target-specific values.
-                if isinstance(part, CompiledPart):
-                    part.defines = self._normalised_values(part.defines)
-                    part.includepath = self._normalised_values(
-                            part.includepath)
-                    part.libs = self._normalised_values(part.libs)
+        # Resolve all target-specific values.
+        if isinstance(part, CompiledPart):
+            part.defines = self._normalised_values(part.defines)
+            part.includepath = self._normalised_values(part.includepath)
+            part.libs = self._normalised_values(part.libs)
 
-                if isinstance(part, ExtensionModule):
-                    part.source = self._normalised_values(part.source)
-
-                break
-        else:
-            part = None
+        if isinstance(part, ExtensionModule):
+            part.source = self._normalised_values(part.source)
 
         return part
 
