@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -44,8 +44,11 @@ class Component(AbstractComponent):
     # The following make up the public API to be used by component plugins.
     ###########################################################################
 
-    # Set if installing from source is mandatory.
-    must_install_from_source = False
+    # Set if installing from source is mandatory.  This is deprecated.
+    must_install_from_source = None
+
+    # Set if the 'install_from_source' option is supported.
+    option_install_from_source = False
 
     def get_archive(self):
         """ Return the pathname of a local copy of a source archive.  The
@@ -100,10 +103,10 @@ class Component(AbstractComponent):
 
     @abstractmethod
     def get_archive_name(self):
-        """ Return the filename of the source archive. """
+        """ Return the filename of the source archive or wheel. """
 
     def get_archive_urls(self):
-        """ Return the list of URLs where the source archive might be
+        """ Return the list of URLs where a source archive or wheel might be
         downloaded from.
         """
 
@@ -189,6 +192,18 @@ class Component(AbstractComponent):
 
         return archive_root
 
+    def unpack_wheel(self, wheel_path):
+        """ Unpack a wheel in the target Python installation's site-packages
+        directory.
+        """
+
+        import zipfile
+
+        self.progress("unpacking '{0}'".format(os.path.basename(wheel_path)))
+
+        with zipfile.ZipFile(wheel_path) as zf:
+            zf.extractall(self.get_component('Python').target_sitepackages_dir)
+
     ###########################################################################
     # The following are not part of the public API used by component plugins.
     ###########################################################################
@@ -200,7 +215,16 @@ class Component(AbstractComponent):
 
         options = super().get_options()
 
-        if not self.must_install_from_source:
+        if self.must_install_from_source is not None:
+            import warnings
+
+            warnings.warn(
+                    "'must_install_from_source' is deprecated, use 'option_install_from_source' instead",
+                    DeprecationWarning, stacklevel=2)
+
+            self.option_install_from_source = not self.must_install_from_source
+
+        if self.option_install_from_source:
             options.append(
                     ComponentOption('install_from_source', type=bool,
                             default=True,
