@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,6 +24,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 
+from importlib import resources
 import os
 import shutil
 import sys
@@ -33,16 +34,23 @@ from .... import AbstractPythonComponent, ComponentOption
 from .pyconfig import generate_pyconfig_h
 from .standard_library import standard_library
 
+from . import configurations as configurations_package
+from .configurations import pyconfig as pyconfig_package
+
 
 # The latest tested patch releases of each minor version.
-LATEST_3_5_RELEASE = (3, 5, 10)
-LATEST_3_6_RELEASE = (3, 6, 11)
-LATEST_3_7_RELEASE = (3, 7, 8)
+LATEST_3_7_RELEASE = (3, 7, 13)
+LATEST_3_8_RELEASE = (3, 8, 13)
+LATEST_3_9_RELEASE = (3, 9, 12)
+LATEST_3_10_RELEASE = (3, 10, 4)
 
 
 class PythonComponent(AbstractPythonComponent):
     """ The host and target Python component. """
 
+    # Add the 'install_from_source' option.
+    option_install_from_source = True
+
     # The list of components that, if specified, should be installed before
     # this one.  Note that we don't need to specify things like 'OpenSSL' as
     # these are only used by this component when building the application and
@@ -174,18 +182,25 @@ class PythonComponent(AbstractPythonComponent):
     def verify(self):
         """ Verify the component. """
 
-        if self.version < (3, 5):
+        if self.version < (3, 7):
             self.unsupported()
 
-        if self.version == (3, 5):
-            if self.version > LATEST_3_5_RELEASE:
-                self.untested()
-        elif self.version == (3, 6):
-            if self.version > LATEST_3_6_RELEASE:
-                self.untested()
-        elif self.version == (3, 7):
+        if self.version == (3, 7):
             if self.version > LATEST_3_7_RELEASE:
                 self.untested()
+        elif self.version == (3, 8):
+            if self.version > LATEST_3_8_RELEASE:
+                self.untested()
+        elif self.version == (3, 9):
+            # This version was withdrawn.
+            if self.version == (3, 9, 3):
+                self.unsupported()
+
+            if self.version > LATEST_3_9_RELEASE:
+                self.untested()
+        elif self.version == (3, 10):
+            if self.version > LATEST_3_10_RELEASE:
+                self.untested()
         else:
             self.unsupported()
 
@@ -213,9 +228,7 @@ class PythonComponent(AbstractPythonComponent):
 
             # Check the OpenSSL support.
             openssl = self.get_component('OpenSSL', required=False)
-            if openssl is None:
-                self._has_openssl = False
-            else:
+            if openssl is not None:
                 if self.version >= (3, 8):
                     if openssl.version < (1, 1, 1):
                         self.error(
@@ -236,24 +249,13 @@ class PythonComponent(AbstractPythonComponent):
                         self.error(
                                 "v{0} requires OpenSSL v1.0.2".format(
                                         self.version))
-
-                self._has_openssl = True
         elif self.host_platform_name != 'win':
             self.error(
                     "using an existing Python installation for the target is "
                     "not supported on {0}".format(self.target_platform_name))
 
-            # A standard Python builds support OpenSSL.
-            self._has_openssl = True
-
-        if self.target_platform_name == 'android':
-            if self.version < (3, 6):
-                self.error(
-                        "v{0} is not supported on Android".format(
-                                self.version))
-
-            if self.android_api < 21:
-                self.error("Android API level 21 or greater is required")
+        if self.target_platform_name == 'android' and self.android_api < 21:
+            self.error("Android API level 21 or greater is required")
 
     def _configure_python(self):
         """ Configure a Python source directory for a particular target. """
@@ -264,17 +266,14 @@ class PythonComponent(AbstractPythonComponent):
 
         py_src_dir = os.getcwd()
 
-        configurations_dir = os.path.join(os.path.dirname(__file__),
-                'configurations')
-
         # Copy the modules config.c file.
         config_c_src_file = 'config_py{0}.c'.format(self.version.major)
         config_c_dst_file = os.path.join(py_src_dir, 'Modules', 'config.c')
 
         self.verbose("installing {0}".format(config_c_dst_file))
 
-        self.copy_file(os.path.join(configurations_dir, config_c_src_file),
-                config_c_dst_file)
+        with resources.path(configurations_package, config_c_src_file) as path:
+            self.copy_file(path, config_c_dst_file)
 
         # Generate the pyconfig.h file.  We follow the Python approach of a
         # static version for Windows and a dynamically created version for
@@ -285,11 +284,10 @@ class PythonComponent(AbstractPythonComponent):
             self.verbose("installing {0}".format(pyconfig_h_dst_file))
 
             # Find the pyconfig.h file appropriate for this version of Python.
-            pyconfig_dir = os.path.join(configurations_dir, 'pyconfig')
             pyconfig = None
             pyconfig_version = None
 
-            for fn in os.listdir(pyconfig_dir):
+            for fn in resources.contents(pyconfig_package):
                 version = fn.split('-')[-1]
                 if version.endswith('.h'):
                     version = version[:-2]
@@ -310,9 +308,10 @@ class PythonComponent(AbstractPythonComponent):
 
             assert pyconfig is not None
 
-            self.copy_file(os.path.join(pyconfig_dir, pyconfig),
-                    pyconfig_h_dst_file, macros={
-                        '@PY_DYNAMIC_LOADING@': '#define' if self.dynamic_loading else '#undef'})
+            with resources.path(pyconfig_package, pyconfig) as path:
+                self.copy_file(path,
+                        pyconfig_h_dst_file, macros={
+                            '@PY_DYNAMIC_LOADING@': '#define' if self.dynamic_loading else '#undef'})
 
             # Rename these otherwise MSVC confuses them with the ones we want
             # to use.
@@ -337,13 +336,14 @@ class PythonComponent(AbstractPythonComponent):
         # This is needed by Qt v5.14 and later on Android.
         android_abis = self.android_abi if self.target_platform_name == 'android' else ''
 
-        self.copy_file(os.path.join(configurations_dir, 'python.pro'),
-                python_pro_dst_file, macros={
-                    '@PY_MAJOR_VERSION@': str(self.version.major),
-                    '@PY_MINOR_VERSION@': str(self.version.minor),
-                    '@PY_PATCH_VERSION@': str(self.version.patch),
-                    '@PY_DYNAMIC_LOADING@': 'enabled' if self.dynamic_loading else 'disabled',
-                    '@ANDROID_ABIS@': android_abis})
+        with resources.path(configurations_package, 'python.pro') as path:
+            self.copy_file(path,
+                    python_pro_dst_file, macros={
+                        '@PY_MAJOR_VERSION@': str(self.version.major),
+                        '@PY_MINOR_VERSION@': str(self.version.minor),
+                        '@PY_PATCH_VERSION@': str(self.version.patch),
+                        '@PY_DYNAMIC_LOADING@': 'enabled' if self.dynamic_loading else 'disabled',
+                        '@ANDROID_ABIS@': android_abis})
 
     def _create_sysconfigdata(self):
         """ Create the _sysconfigdata module. """
@@ -488,6 +488,15 @@ build_time_vars = {
             self.patch_file(os.path.join('Modules', 'posixmodule.c'),
                     self._patch_for_ios_system)
 
+        elif self.target_platform_name == 'macos':
+            # We use the macOS libffi from Python v3.8.10.
+            if self.version >= (3, 8, 10):
+                ctypes_dir = os.path.join('Modules', '_ctypes')
+
+                for src in ('_ctypes.c', 'callbacks.c', 'callproc.c', 'stgdict.c', 'cfield.c', 'malloc_closure.c'):
+                    self.patch_file(os.path.join(ctypes_dir, src),
+                            self._patch_for_macos_ctypes)
+
         elif self.target_platform_name == 'win':
             # If we are supporting dynamic loading then we must be being built
             # as a DLL.
@@ -495,9 +504,6 @@ build_time_vars = {
                 self.patch_file(os.path.join('Lib', 'ctypes', '__init__.py'),
                         self._patch_for_win_ctypes)
 
-            self.patch_file(os.path.join('Modules', '_io', '_iomodule.c'),
-                    self._patch_for_win_iomodule)
-
             self.patch_file(
                     os.path.join('Modules', 'expat', 'winconfig.h'),
                     self._patch_for_win_expat)
@@ -522,6 +528,14 @@ build_time_vars = {
         if minimal != '#defineHAVE_SYSTEM1':
             patch_file.write(line)
 
+    @staticmethod
+    def _patch_for_macos_ctypes(line, patch_file):
+        """ ctypes specifies <ffi.h> in various files but macOS needs
+        <ffi/ffi.h>.
+        """
+
+        patch_file.write(line.replace('<ffi.h>', '<ffi/ffi.h>'))
+
     @staticmethod
     def _patch_for_win_expat(line, patch_file):
         """ Python.h needs to be included before windows.h.  A regular build
@@ -544,14 +558,6 @@ build_time_vars = {
 
         patch_file.write(line.replace('_sys.dllhandle', '0'))
 
-    @staticmethod
-    def _patch_for_win_iomodule(line, patch_file):
-        """ _iomodule.c in Python v3.6 includes consoleapi.h when it should
-        include windows.h (as it does in Python v3.7).
-        """
-
-        patch_file.write(line.replace('consoleapi.h', 'windows.h'))
-
     @staticmethod
     def _patch_for_win_winapi(line, patch_file):
         """ Both _winapi.c and overlapped.c define a C structure with the name
