@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,13 @@ from ... import Component, ComponentLibrary, ComponentOption
 class zlibComponent(Component):
     """ The zlib component. """
 
+    # Add the 'install_from_source' option.
+    option_install_from_source = True
+
+    # The version will be determined dynamically if the system provided version
+    # is being used.
+    version_is_optional = True
+
     def get_archive_name(self):
         """ Return the filename of the source archive. """
 
@@ -92,8 +99,7 @@ class zlibComponent(Component):
 
             self.run('./configure', '--static', '--prefix=' + self.sysroot_dir)
             self.run(self.host_make,
-                    'AR=' + self.android_toolchain_prefix + 'ar cqs',
-                    'install')
+                    'AR=' + self.android_toolchain_prefix + 'ar', 'install')
 
             if self.target_arch_name == 'android-32':
                 del os.environ['CFLAGS']
@@ -124,14 +130,23 @@ class zlibComponent(Component):
     def verify(self):
         """ Verify the component. """
 
-        # Make sure any installed version is the one specified.
-        if not self.install_from_source:
-            self._verify_installed_version()
+        if self.install_from_source:
+            if self.version is None:
+                self.error(
+                        "'version' must be specified when installing from "
+                        "source")
+        else:
+            installed_version = self._get_installed_version()
 
-    def _verify_installed_version(self):
-        """ Verify that the installed version is compatible with the specified
-        version.
-        """
+            if self.version is None:
+                self.version = installed_version
+            elif self.version != installed_version:
+                self.error(
+                        "v{0} is specified but the host installation is "
+                                "v{1}".format(self.version, installed_version))
+
+    def _get_installed_version(self):
+        """ Get the installed version. """
 
         # We support native versions for everything except Windows.
         if self.target_platform_name == 'android':
@@ -154,9 +169,4 @@ class zlibComponent(Component):
         if version_str.endswith('"'):
             version_str = version_str[:-1]
 
-        installed_version = self.parse_version_number(version_str)
-
-        if self.version != installed_version:
-            self.error(
-                    "v{0} is specified but the host installation is "
-                            "v{1}".format(self.version, installed_version))
+        return self.parse_version_number(version_str)
